package xhttp3.internal.http;

import java.io.EOFException;
import java.io.IOException;
import java.net.ProtocolException;
import java.util.concurrent.TimeUnit;
import xhttp3.Headers;
import xhttp3.HttpUrl;
import xhttp3.OkHttpClient;
import xhttp3.Request;
import xhttp3.Response;
import xhttp3.ResponseBody;
import xhttp3.internal.Internal;
import xhttp3.internal.Util;
import xhttp3.internal.connection.RealConnection;
import xhttp3.internal.connection.StreamAllocation;
import xio.Buffer;
import xio.BufferedSink;
import xio.BufferedSource;
import xio.ForwardingTimeout;
import xio.Okio;
import xio.Sink;
import xio.Source;
import xio.Timeout;

public final class Http1xStream implements HttpStream {
   private static final int STATE_IDLE = 0;
   private static final int STATE_OPEN_REQUEST_BODY = 1;
   private static final int STATE_WRITING_REQUEST_BODY = 2;
   private static final int STATE_READ_RESPONSE_HEADERS = 3;
   private static final int STATE_OPEN_RESPONSE_BODY = 4;
   private static final int STATE_READING_RESPONSE_BODY = 5;
   private static final int STATE_CLOSED = 6;
   private final OkHttpClient client;
   private final StreamAllocation streamAllocation;
   private final BufferedSource source;
   private final BufferedSink sink;
   private int state = 0;

   public Http1xStream(OkHttpClient client, StreamAllocation streamAllocation, BufferedSource source, BufferedSink sink) {
      this.client = client;
      this.streamAllocation = streamAllocation;
      this.source = source;
      this.sink = sink;
   }

   private Source getTransferStream(Response response) {
      if (!HttpHeaders.hasBody(response)) {
         return this.newFixedLengthSource(0L);
      } else if ("chunked".equalsIgnoreCase(response.header("Transfer-Encoding"))) {
         return this.newChunkedSource(response.request().url());
      } else {
         long var2;
         return (var2 = HttpHeaders.contentLength(response)) != -1L ? this.newFixedLengthSource(var2) : this.newUnknownLengthSource();
      }
   }

   private void detachTimeout(ForwardingTimeout timeout) {
      Timeout var10000 = timeout.delegate();
      timeout.setDelegate(Timeout.NONE);
      var10000.clearDeadline();
      var10000.clearTimeout();
   }

   public Sink createRequestBody(Request request, long contentLength) {
      if ("chunked".equalsIgnoreCase(request.header("Transfer-Encoding"))) {
         return this.newChunkedSink();
      } else if (contentLength != -1L) {
         return this.newFixedLengthSink(contentLength);
      } else {
         throw new IllegalStateException("Cannot stream a request body without chunked encoding or a known content length!");
      }
   }

   public void cancel() {
      RealConnection var1;
      if ((var1 = this.streamAllocation.connection()) != null) {
         var1.cancel();
      }

   }

   public void writeRequestHeaders(Request request) {
      Http1xStream var10000 = this;
      String var2 = RequestLine.get(request, this.streamAllocation.connection().route().proxy().type());
      var10000.writeRequest(request.headers(), var2);
   }

   public Response.Builder readResponseHeaders() {
      return this.readResponse();
   }

   public ResponseBody openResponseBody(Response response) {
      Source var2 = this.getTransferStream(response);
      Source var10002 = var2;
      Headers var3 = response.headers();
      return new RealResponseBody(var3, Okio.buffer(var10002));
   }

   public boolean isClosed() {
      return this.state == 6;
   }

   public void finishRequest() {
      this.sink.flush();
   }

   public void writeRequest(Headers headers, String requestLine) {
      if (this.state != 0) {
         throw new IllegalStateException("state: " + this.state);
      } else {
         this.sink.writeUtf8(requestLine).writeUtf8("\r\n");
         int requestLine1 = 0;

         for(int var3 = headers.size(); requestLine1 < var3; ++requestLine1) {
            this.sink.writeUtf8(headers.name(requestLine1)).writeUtf8(": ").writeUtf8(headers.value(requestLine1)).writeUtf8("\r\n");
         }

         this.sink.writeUtf8("\r\n");
         this.state = 1;
      }
   }

   public Response.Builder readResponse() {
      int var1;
      if ((var1 = this.state) != 1 && var1 != 3) {
         throw new IllegalStateException("state: " + this.state);
      } else {
         EOFException var10000;
         while(true) {
            StatusLine var10;
            boolean var10001;
            try {
               var10 = StatusLine.parse(this.source.readUtf8LineStrict());
            } catch (EOFException var6) {
               var10000 = var6;
               var10001 = false;
               break;
            }

            StatusLine var7 = var10;

            Response.Builder var11;
            try {
               var11 = new Response.Builder;
            } catch (EOFException var5) {
               var10000 = var5;
               var10001 = false;
               break;
            }

            Response.Builder var10002 = var11;

            try {
               var10002.<init>();
               var11 = var11.protocol(var7.protocol).code(var7.code).message(var7.message).headers(this.readHeaders());
            } catch (EOFException var4) {
               var10000 = var4;
               var10001 = false;
               break;
            }

            Response.Builder var8 = var11;

            int var12;
            try {
               var12 = var10.code;
            } catch (EOFException var3) {
               var10000 = var3;
               var10001 = false;
               break;
            }

            if (var12 != 100) {
               try {
                  this.state = 4;
                  return var8;
               } catch (EOFException var2) {
                  var10000 = var2;
                  var10001 = false;
                  break;
               }
            }
         }

         EOFException var9 = var10000;
         IOException var13 = new IOException("unexpected end of stream on " + this.streamAllocation);
         var13.initCause(var9);
         throw var13;
      }
   }

   public Headers readHeaders() {
      Headers.Builder var1;
      var1 = new Headers.Builder.<init>();

      String var2;
      while((var2 = this.source.readUtf8LineStrict()).length() != 0) {
         Internal.instance.addLenient(var1, var2);
      }

      return var1.build();
   }

   public Sink newChunkedSink() {
      if (this.state == 1) {
         this.state = 2;
         return new ChunkedSink();
      } else {
         throw new IllegalStateException("state: " + this.state);
      }
   }

   public Sink newFixedLengthSink(long contentLength) {
      if (this.state == 1) {
         this.state = 2;
         return new FixedLengthSink(contentLength);
      } else {
         throw new IllegalStateException("state: " + this.state);
      }
   }

   public Source newFixedLengthSource(long length) {
      if (this.state == 4) {
         this.state = 5;
         return new FixedLengthSource(length);
      } else {
         throw new IllegalStateException("state: " + this.state);
      }
   }

   public Source newChunkedSource(HttpUrl url) {
      if (this.state == 4) {
         this.state = 5;
         return new ChunkedSource(url);
      } else {
         throw new IllegalStateException("state: " + this.state);
      }
   }

   public Source newUnknownLengthSource() {
      if (this.state == 4) {
         StreamAllocation var1;
         if ((var1 = this.streamAllocation) != null) {
            this.state = 5;
            var1.noNewStreams();
            return new UnknownLengthSource();
         } else {
            throw new IllegalStateException("streamAllocation == null");
         }
      } else {
         throw new IllegalStateException("state: " + this.state);
      }
   }

   private class UnknownLengthSource extends AbstractSource {
      private boolean inputExhausted;

      private UnknownLengthSource() {
         super(null);
      }

      // $FF: synthetic method
      UnknownLengthSource(Object x1) {
         this();
      }

      public long read(Buffer sink, long byteCount) {
         if (byteCount >= 0L) {
            if (!super.closed) {
               if (this.inputExhausted) {
                  return -1L;
               } else {
                  long sink1;
                  if ((sink1 = Http1xStream.this.source.read(sink, byteCount)) == -1L) {
                     this.inputExhausted = true;
                     this.endOfInput(true);
                     return -1L;
                  } else {
                     return sink1;
                  }
               }
            } else {
               throw new IllegalStateException("closed");
            }
         } else {
            throw new IllegalArgumentException("byteCount < 0: " + byteCount);
         }
      }

      public void close() {
         if (!super.closed) {
            if (!this.inputExhausted) {
               this.endOfInput(false);
            }

            super.closed = true;
         }
      }
   }

   private class ChunkedSource extends AbstractSource {
      private static final long NO_CHUNK_YET = -1L;
      private final HttpUrl url;
      private long bytesRemainingInChunk = -1L;
      private boolean hasMoreChunks = true;

      ChunkedSource(HttpUrl url) {
         super(null);
         this.url = url;
      }

      private void readChunkSize() {
         if (this.bytesRemainingInChunk != -1L) {
            Http1xStream.this.source.readUtf8LineStrict();
         }

         NumberFormatException var10000;
         label60: {
            ChunkedSource var8;
            String var10;
            boolean var10001;
            try {
               var8 = this;
               this.bytesRemainingInChunk = Http1xStream.this.source.readHexadecimalUnsignedLong();
               var10 = Http1xStream.this.source.readUtf8LineStrict().trim();
            } catch (NumberFormatException var6) {
               var10000 = var6;
               var10001 = false;
               break label60;
            }

            String var1 = var10;

            long var9;
            try {
               var9 = var8.bytesRemainingInChunk;
            } catch (NumberFormatException var5) {
               var10000 = var5;
               var10001 = false;
               break label60;
            }

            if (var9 >= 0L) {
               label59: {
                  boolean var11;
                  try {
                     var11 = var1.isEmpty();
                  } catch (NumberFormatException var4) {
                     var10000 = var4;
                     var10001 = false;
                     break label60;
                  }

                  if (!var11) {
                     try {
                        var11 = var1.startsWith(";");
                     } catch (NumberFormatException var3) {
                        var10000 = var3;
                        var10001 = false;
                        break label60;
                     }

                     if (!var11) {
                        break label59;
                     }
                  }

                  if (this.bytesRemainingInChunk == 0L) {
                     this.hasMoreChunks = false;
                     HttpHeaders.receiveHeaders(Http1xStream.this.client.cookieJar(), this.url, Http1xStream.this.readHeaders());
                     this.endOfInput(true);
                  }

                  return;
               }
            }

            try {
               throw new ProtocolException("expected chunk size and optional extensions but was \"" + this.bytesRemainingInChunk + var1 + "\"");
            } catch (NumberFormatException var2) {
               var10000 = var2;
               var10001 = false;
            }
         }

         NumberFormatException var7 = var10000;
         throw new ProtocolException(var7.getMessage());
      }

      public long read(Buffer sink, long byteCount) {
         if (byteCount < 0L) {
            throw new IllegalArgumentException("byteCount < 0: " + byteCount);
         } else if (!super.closed) {
            if (!this.hasMoreChunks) {
               return -1L;
            } else {
               long var4;
               if ((var4 = this.bytesRemainingInChunk) == 0L || var4 == -1L) {
                  this.readChunkSize();
                  if (!this.hasMoreChunks) {
                     return -1L;
                  }
               }

               long sink1;
               if ((sink1 = Http1xStream.this.source.read(sink, Math.min(byteCount, this.bytesRemainingInChunk))) != -1L) {
                  this.bytesRemainingInChunk -= sink1;
                  return sink1;
               } else {
                  this.endOfInput(false);
                  throw new ProtocolException("unexpected end of stream");
               }
            }
         } else {
            throw new IllegalStateException("closed");
         }
      }

      public void close() {
         if (!super.closed) {
            if (this.hasMoreChunks && !Util.discard(this, 100, TimeUnit.MILLISECONDS)) {
               this.endOfInput(false);
            }

            super.closed = true;
         }
      }
   }

   private class FixedLengthSource extends AbstractSource {
      private long bytesRemaining;

      public FixedLengthSource(long length) {
         super(null);
         this.bytesRemaining = length;
         if (length == 0L) {
            this.endOfInput(true);
         }

      }

      public long read(Buffer sink, long byteCount) {
         if (byteCount >= 0L) {
            if (!super.closed) {
               if (this.bytesRemaining == 0L) {
                  return -1L;
               } else {
                  long sink1;
                  if ((sink1 = Http1xStream.this.source.read(sink, Math.min(this.bytesRemaining, byteCount))) != -1L) {
                     long var4;
                     long var10000 = var4 = this.bytesRemaining - sink1;
                     this.bytesRemaining = var4;
                     if (var10000 == 0L) {
                        this.endOfInput(true);
                     }

                     return sink1;
                  } else {
                     this.endOfInput(false);
                     throw new ProtocolException("unexpected end of stream");
                  }
               }
            } else {
               throw new IllegalStateException("closed");
            }
         } else {
            throw new IllegalArgumentException("byteCount < 0: " + byteCount);
         }
      }

      public void close() {
         if (!super.closed) {
            if (this.bytesRemaining != 0L && !Util.discard(this, 100, TimeUnit.MILLISECONDS)) {
               this.endOfInput(false);
            }

            super.closed = true;
         }
      }
   }

   private abstract class AbstractSource implements Source {
      protected final ForwardingTimeout timeout;
      protected boolean closed;

      private AbstractSource() {
         this.timeout = new ForwardingTimeout(Http1xStream.this.source.timeout());
      }

      // $FF: synthetic method
      AbstractSource(Object x1) {
         this();
      }

      public Timeout timeout() {
         return this.timeout;
      }

      protected final void endOfInput(boolean reuseConnection) {
         if (Http1xStream.this.state != 6) {
            if (Http1xStream.this.state == 5) {
               Http1xStream.this.detachTimeout(this.timeout);
               Http1xStream.this.state = 6;
               if (Http1xStream.this.streamAllocation != null) {
                  StreamAllocation var10000 = Http1xStream.this.streamAllocation;
                  AbstractSource var10001 = this;
                  boolean var2 = reuseConnection ^ true;
                  var10000.streamFinished(var2, Http1xStream.this);
               }

            } else {
               throw new IllegalStateException("state: " + Http1xStream.this.state);
            }
         }
      }
   }

   private final class ChunkedSink implements Sink {
      private final ForwardingTimeout timeout;
      private boolean closed;

      private ChunkedSink() {
         this.timeout = new ForwardingTimeout(Http1xStream.this.sink.timeout());
      }

      // $FF: synthetic method
      ChunkedSink(Object x1) {
         this();
      }

      public Timeout timeout() {
         return this.timeout;
      }

      public void write(Buffer source, long byteCount) {
         if (!this.closed) {
            if (byteCount != 0L) {
               Http1xStream.this.sink.writeHexadecimalUnsignedLong(byteCount);
               Http1xStream.this.sink.writeUtf8("\r\n");
               Http1xStream.this.sink.write(source, byteCount);
               Http1xStream.this.sink.writeUtf8("\r\n");
            }
         } else {
            throw new IllegalStateException("closed");
         }
      }

      public synchronized void flush() {
         if (!this.closed) {
            Http1xStream.this.sink.flush();
         }
      }

      public synchronized void close() {
         if (!this.closed) {
            this.closed = true;
            Http1xStream.this.sink.writeUtf8("0\r\n\r\n");
            Http1xStream.this.detachTimeout(this.timeout);
            Http1xStream.this.state = 3;
         }
      }
   }

   private final class FixedLengthSink implements Sink {
      private final ForwardingTimeout timeout;
      private boolean closed;
      private long bytesRemaining;

      private FixedLengthSink(long bytesRemaining) {
         this.timeout = new ForwardingTimeout(Http1xStream.this.sink.timeout());
         this.bytesRemaining = bytesRemaining;
      }

      // $FF: synthetic method
      FixedLengthSink(long x1, Object x2) {
         this(x1);
      }

      public Timeout timeout() {
         return this.timeout;
      }

      public void write(Buffer source, long byteCount) {
         if (!this.closed) {
            Util.checkOffsetAndCount(source.size(), 0L, byteCount);
            if (byteCount <= this.bytesRemaining) {
               Http1xStream.this.sink.write(source, byteCount);
               this.bytesRemaining -= byteCount;
            } else {
               throw new ProtocolException("expected " + this.bytesRemaining + " bytes but received " + byteCount);
            }
         } else {
            throw new IllegalStateException("closed");
         }
      }

      public void flush() {
         if (!this.closed) {
            Http1xStream.this.sink.flush();
         }
      }

      public void close() {
         if (!this.closed) {
            this.closed = true;
            if (this.bytesRemaining <= 0L) {
               Http1xStream.this.detachTimeout(this.timeout);
               Http1xStream.this.state = 3;
            } else {
               throw new ProtocolException("unexpected end of stream");
            }
         }
      }
   }
}
